"""Event handling and generation.

The Event module manages event generation. Events are created by
EventSource instances. Each event source is associated with a file
handle. This handle will be poll()'ed and the eventsource will be
called, as sonn as input is available."""

import select, Logger, Node

class _Event(tuple):

    def __init__(self, elts):
        tuple.__init__(self, elts)

    context = property(lambda self: self[0], None, None)
    code = property(lambda self: self[1], None, None)


def Event(context, code):
    return _Event((context,code))


class EventSource(Node.Node):

    def __init__(self, fd, context):
    	Node.Node.__init__(self)
        self._fd = fd
        self._context = context

    def fd(self):
        return self._fd

    def context(self):
        return self._context

    def fileno(self):
        return self._fd.fileno()

    def readEvents(self):
        raise NotImplementedError


class Dispatcher(Node.Node):

    _sources = Node.NodeMap()

    def __init__(self, keylist):
    	Node.Node.__init__(self)
	self._sources = {}
        self._callbacks = []
        self._keylist = keylist
        self._poller = select.poll()

    def registerSource(self, eventSource):
        self._sources[eventSource.fileno()] = eventSource
	self._poller.register(eventSource.fileno(),select.POLLIN)

    def unregisterSource(self, eventSource):
	del self._sources[eventSource.fileno()]
	self._poller.unregister(eventSource.fileno())

    def registerCallback(self,cb):
        self._callbacks.append(cb)

    def unregisterCallback(self,cb):
        self._callbacks.remove(cb)

    def run(self):
        while 1:
            for cb in self._callbacks : cb()
            try:
                pollEvents = self._poller.poll()
            except select.error, v:
                if v[0]==4 : continue
                else       : raise
            if not pollEvents : return
            for fd, pollEvent in pollEvents:
                if pollEvent != select.POLLIN : return
                for event in  self._sources[fd].readEvents():
                    Logger.log('dispatcher', 'event: ' + str(event))
                    binding = self._keylist.lookup(event)
                    if binding is not None:
                        binding.execute()
