import Views, Events, Logger
from Views import EventWidget
import time, os, struct

class View(Views.WidgetView):

    def __init__(self, context, label, numeric_switches, alpha_switches, x, y, size=11):
        size += 1
        Views.WidgetView.__init__(self, context, label, x, y,size*numeric_switches+3,7)

        split = numeric_switches // 2
        for i in range(split):
            self.add( EventWidget(i,str(i),size*i+1,4,size) )
        for i in range(split,numeric_switches):
            self.add( EventWidget(i,str(i),size*i+3,4,size) )
        split = max(0,alpha_switches-(numeric_switches-split))
        offset = size//2+(numeric_switches-alpha_switches-1)*size
        for i in range(split):
            self.add( EventWidget(i+numeric_switches, chr(ord('A')+i), size*i+1+offset,1,size) )
        for i in range(split, alpha_switches):
            self.add( EventWidget(i+numeric_switches, chr(ord('A')+i), size*i+3+offset,1,size) )


class JSEvent:

    STRUCT = "IhBB"
    STRUCT_len = struct.calcsize(STRUCT)

    TYPE_BUTTON = 0x01
    TYPE_AXIS = 0x02
    TYPE_INIT = 0x80
    
    def __init__(self, data):
        self.time, self.value, self.type, self.number = struct.unpack(self.STRUCT,data)

    def __str__(self):
        return "%s(time=%d, value=%d,type=%d (%s)), number=%d)" % (
            str(self.__class__),self.time, self.value, self.type, self.decode_type(), self.number)

    def decode_type(self):
        return ('','INIT|')[(self.type & self.TYPE_INIT) and 1 or 0] \
               + ('','BUTTON','AXIS')[self.type & ~self.TYPE_INIT]

    def readFrom(klass,f):
        return klass(os.read(f.fileno(),klass.STRUCT_len))
    readFrom=classmethod(readFrom)

    def readMultipleFrom(klass,f,maxevents=256):
        data = os.read(f.fileno(),maxevents*klass.STRUCT_len)
        rv = []
        while data:
            rv.append(klass(data[:klass.STRUCT_len]))
            data = data[klass.STRUCT_len:]
        return rv
    readMultipleFrom=classmethod(readMultipleFrom)

class Source(Events.EventSource):

    def __init__(self, joydev, context, bits=None, mindelay=100):
        Events.EventSource.__init__(self, file(joydev), context)
        self._bits = bits
        self._lastevent = 0
        self._mindelay = mindelay

    def readEvents(self):
        n = 0
        lev = self._lastevent
        if self._bits:
            time.sleep(0.0005)
            jsevents = JSEvent.readMultipleFrom(self.fd())
            for event in jsevents:
                if event.type == JSEvent.TYPE_BUTTON and event.value == 1:
                    self._lastevent = event.time
                    if event.time - lev < self._mindelay : return []
                    n = n | self._bits[event.number]
            if n == 0 : return []
            n -= 1
        else:
            event = JSEvent.readFrom(self.fd())
            if event.type == JSEvent.TYPE_BUTTON and event.value == 1:
                self._lastevent = event.time
                if event.time - lev < self._mindelay : return []
                n = event.number
            else:
                return []
        return [Events.Event(self.context(), n)]
    
def register( viewmanager,
              dispatcher,
              context,
              label,
              numeric_switches,
              alpha_switches,
              x,
              y,
              size,
              device,
              bits = None ):
    viewmanager.registerView( View(context, label, numeric_switches, alpha_switches, x, y, size) )
    dispatcher.registerSource( Source(device, context, bits) )
