import Bindings, Actions, Views, Events, Logger
from Bindings import Binding
from Actions import Actions as Action, action
from Views import EventWidget
from Events import Event
import main
import Joyboard, Keyboard, Process
import Looper
import sys, curses
from curses.ascii import alt, ctrl

def shift(letter) : return ord(chr(letter).upper())
def key(letter) : return ord(letter.lower())

###########################################################################
# Setup views and controllers
#
# Display size: 77x17

Logger.init(main.viewmanager, 38, 0, 39, 10)

Joyboard.register(
    viewmanager = main.viewmanager,
    dispatcher = main.dispatcher,

    context          = 'jb0',
    label            = 'Foot Switch',
    numeric_switches =  10,
    alpha_switches   =   5,
    x                =   2,
    y                =  10,
    size             =   6,

    device           = '/dev/input/js0',
    bits             = { 1:1, 3:2, 2:4, 0:8 }
)

Keyboard.register(
    viewmanager = main.viewmanager,
    dispatcher = main.dispatcher,

    context          = 'kbd',
    label            = 'Key Bindings',
    x                =  0,
    y                =  0,
    dx               = 38,
    dy               = 10
)

jackd = Process.Process("jackd -d alsa -S -r 44100 -p 256",'jack')

###########################################################################
# Global keymap and auxilary actions

global_map = Bindings.KeyMap()

@action
def quit(binding):
    sys.exit(0)

global_map.add( Binding( Event('jb0',0), '()', Actions.Nop() ) )
global_map.add( Binding( Event('jb0',1), '()', Actions.Nop() ) )
global_map.add( Binding( Event('jb0',10), '()', Actions.Nop() ) )

global_map.add( Binding( Event('kbd',key('q')), 'Quit', Action['quit'] ) )

Action.register( Actions.ChangeBindingsRelative( 'unset_this_map', 0, [] ) )

###########################################################################
# Looper mode

looper_main_map = Bindings.KeyMap( 'Looper' )
looper_aux_map = Bindings.KeyMap( 'Aux' )

Action.register( Actions.ChangeBindingsAbsolute('set_mode_looper', 1, [looper_main_map]) )
Action.register( Actions.ChangeBindingsRelative('looper_set_aux_map', 1, [looper_aux_map]) )

looper = Looper.Controller('localhost',9951)

Action.register( Looper.Command('looper_record', looper, 'record') )
Action.register( Looper.Command('looper_overdub', looper, 'overdub') )
Action.register( Looper.Command('looper_multiply', looper, 'multiply') )
Action.register( Looper.Command('looper_mute', looper, 'mute') )
Action.register( Looper.Command('looper_undo', looper, 'undo') )
Action.register( Looper.Command('looper_redo', looper, 'redo') )
Action.register( Looper.Command('looper_trigger', looper, 'trigger') )
Action.register( Looper.Command('looper_insert', looper, 'insert') )
Action.register( Looper.Command('looper_replace', looper, 'replace') )
Action.register( Looper.Command('looper_substitute', looper, 'substitute') )
Action.register( Looper.Command('looper_reverse', looper, 'reverse') )
Action.register( Looper.Command('looper_oneshot', looper, 'oneshot') )

looper_main_map.add ( Binding( Event('jb0',2),  'Rec',     Action['looper_record'] ) )
looper_main_map.add ( Binding( Event('jb0',3),  'Over',    Action['looper_overdub'] ) )
looper_main_map.add ( Binding( Event('jb0',4),  'Mult',   Action['looper_multiply'] ) )
looper_main_map.add ( Binding( Event('jb0',5),  'Mute',       Action['looper_mute'] ) )
looper_main_map.add ( Binding( Event('jb0',6),  'Undo',       Action['looper_undo'] ) )
looper_main_map.add ( Binding( Event('jb0',7),  'Redo',       Action['looper_redo'] ) )
looper_main_map.add ( Binding( Event('jb0',8),  'Trig',    Action['looper_trigger'] ) )
looper_main_map.add ( Binding( Event('jb0',9),  'Aux',        Action['looper_set_aux_map'] ) )

looper_aux_map.add  ( Binding( Event('jb0',2),  'Ins',     Action['looper_insert'] ) )
looper_aux_map.add  ( Binding( Event('jb0',3),  'Repl',    Action['looper_replace'] ) )
looper_aux_map.add  ( Binding( Event('jb0',4),  'Subst', Action['looper_substitute'] ) )
looper_aux_map.add  ( Binding( Event('jb0',6),  'Rev',    Action['looper_reverse'] ) )
looper_aux_map.add  ( Binding( Event('jb0',7),  'Once',    Action['looper_oneshot'] ) )
looper_aux_map.add  ( Binding( Event('jb0',9),  'Main',       Action['unset_this_map'] ) )

###########################################################################
# MegaPedal mode

megapedal_main_map = Bindings.KeyMap( 'MegaPedal' )

Action.register( Actions.ChangeBindingsAbsolute('set_mode_megapedal', 1, [megapedal_main_map]) )

megapedal_main_map.add ( Binding( Event('jb0',11), 'Looper', Action['set_mode_looper'] ) )
global_map.add ( Binding( Event('jb0',11), 'MegaPedal', Action['set_mode_megapedal'] ) )

###########################################################################
# RythmBox mode

rythmbox_main_map = Bindings.KeyMap( 'RythmBox' )

Action.register( Actions.ChangeBindingsAbsolute('set_mode_rythmbox', 1, [rythmbox_main_map]) )

rythmbox_main_map.add ( Binding( Event('jb0',11), 'MegaPedal', Action['set_mode_megapedal'] ) )
global_map.add ( Binding( Event('jb0',11), 'RythmBox', Action['set_mode_rythmbox'] ) )

###########################################################################

main.keylist.append(global_map)
main.keylist.append(looper_main_map)
